import _plotly_utils.basevalidators


class UirevisionValidator(_plotly_utils.basevalidators.AnyValidator):
    def __init__(self, plotly_name="uirevision", parent_name="layout.geo", **kwargs):
        super(UirevisionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class SubunitwidthValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="subunitwidth", parent_name="layout.geo", **kwargs):
        super(SubunitwidthValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class SubunitcolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="subunitcolor", parent_name="layout.geo", **kwargs):
        super(SubunitcolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowsubunitsValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showsubunits", parent_name="layout.geo", **kwargs):
        super(ShowsubunitsValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowriversValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showrivers", parent_name="layout.geo", **kwargs):
        super(ShowriversValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowoceanValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showocean", parent_name="layout.geo", **kwargs):
        super(ShowoceanValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowlandValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showland", parent_name="layout.geo", **kwargs):
        super(ShowlandValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowlakesValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showlakes", parent_name="layout.geo", **kwargs):
        super(ShowlakesValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowframeValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showframe", parent_name="layout.geo", **kwargs):
        super(ShowframeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowcountriesValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showcountries", parent_name="layout.geo", **kwargs):
        super(ShowcountriesValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowcoastlinesValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(
        self, plotly_name="showcoastlines", parent_name="layout.geo", **kwargs
    ):
        super(ShowcoastlinesValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ScopeValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="scope", parent_name="layout.geo", **kwargs):
        super(ScopeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            values=kwargs.pop(
                "values",
                [
                    "world",
                    "usa",
                    "europe",
                    "asia",
                    "africa",
                    "north america",
                    "south america",
                ],
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class RiverwidthValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="riverwidth", parent_name="layout.geo", **kwargs):
        super(RiverwidthValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class RivercolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="rivercolor", parent_name="layout.geo", **kwargs):
        super(RivercolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ResolutionValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="resolution", parent_name="layout.geo", **kwargs):
        super(ResolutionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            coerce_number=kwargs.pop("coerce_number", True),
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            values=kwargs.pop("values", [110, 50]),
            **kwargs
        )


import _plotly_utils.basevalidators


class ProjectionValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="projection", parent_name="layout.geo", **kwargs):
        super(ProjectionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Projection"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            parallels
                For conic projection types only. Sets the
                parallels (tangent, secant) where the cone
                intersects the sphere.
            rotation
                plotly.graph_objects.layout.geo.projection.Rota
                tion instance or dict with compatible
                properties
            scale
                Zooms in or out on the map view. A scale of 1
                corresponds to the largest zoom level that fits
                the map's lon and lat ranges.
            type
                Sets the projection type.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class OceancolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="oceancolor", parent_name="layout.geo", **kwargs):
        super(OceancolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class LonaxisValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="lonaxis", parent_name="layout.geo", **kwargs):
        super(LonaxisValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Lonaxis"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            dtick
                Sets the graticule's longitude/latitude tick
                step.
            gridcolor
                Sets the graticule's stroke color.
            gridwidth
                Sets the graticule's stroke width (in px).
            range
                Sets the range of this axis (in degrees), sets
                the map's clipped coordinates.
            showgrid
                Sets whether or not graticule are shown on the
                map.
            tick0
                Sets the graticule's starting tick
                longitude/latitude.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class LataxisValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="lataxis", parent_name="layout.geo", **kwargs):
        super(LataxisValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Lataxis"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            dtick
                Sets the graticule's longitude/latitude tick
                step.
            gridcolor
                Sets the graticule's stroke color.
            gridwidth
                Sets the graticule's stroke width (in px).
            range
                Sets the range of this axis (in degrees), sets
                the map's clipped coordinates.
            showgrid
                Sets whether or not graticule are shown on the
                map.
            tick0
                Sets the graticule's starting tick
                longitude/latitude.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class LandcolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="landcolor", parent_name="layout.geo", **kwargs):
        super(LandcolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class LakecolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="lakecolor", parent_name="layout.geo", **kwargs):
        super(LakecolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class FramewidthValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="framewidth", parent_name="layout.geo", **kwargs):
        super(FramewidthValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class FramecolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="framecolor", parent_name="layout.geo", **kwargs):
        super(FramecolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class DomainValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="domain", parent_name="layout.geo", **kwargs):
        super(DomainValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Domain"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            column
                If there is a layout grid, use the domain for
                this column in the grid for this geo subplot .
                Note that geo subplots are constrained by
                domain. In general, when `projection.scale` is
                set to 1. a map will fit either its x or y
                domain, but not both.
            row
                If there is a layout grid, use the domain for
                this row in the grid for this geo subplot .
                Note that geo subplots are constrained by
                domain. In general, when `projection.scale` is
                set to 1. a map will fit either its x or y
                domain, but not both.
            x
                Sets the horizontal domain of this geo subplot
                (in plot fraction). Note that geo subplots are
                constrained by domain. In general, when
                `projection.scale` is set to 1. a map will fit
                either its x or y domain, but not both.
            y
                Sets the vertical domain of this geo subplot
                (in plot fraction). Note that geo subplots are
                constrained by domain. In general, when
                `projection.scale` is set to 1. a map will fit
                either its x or y domain, but not both.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class CountrywidthValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="countrywidth", parent_name="layout.geo", **kwargs):
        super(CountrywidthValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class CountrycolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="countrycolor", parent_name="layout.geo", **kwargs):
        super(CountrycolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class CoastlinewidthValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(
        self, plotly_name="coastlinewidth", parent_name="layout.geo", **kwargs
    ):
        super(CoastlinewidthValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class CoastlinecolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(
        self, plotly_name="coastlinecolor", parent_name="layout.geo", **kwargs
    ):
        super(CoastlinecolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class CenterValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="center", parent_name="layout.geo", **kwargs):
        super(CenterValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Center"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            lat
                Sets the latitude of the map's center. For all
                projection types, the map's latitude center
                lies at the middle of the latitude range by
                default.
            lon
                Sets the longitude of the map's center. By
                default, the map's longitude center lies at the
                middle of the longitude range for scoped
                projection and above `projection.rotation.lon`
                otherwise.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class BgcolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="bgcolor", parent_name="layout.geo", **kwargs):
        super(BgcolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )



import inspect

colref = "(string: name of column in `data_frame`)"
colref_list = "(list of string: names of columns in `data_frame`)"

# TODO contents of columns
# TODO explain categorical
# TODO handle color
# TODO handle details of box/violin/histogram
# TODO handle details of column selection with `dimensions`
# TODO document "or `None`, default `None`" in various places
# TODO standardize positioning and casing of 'default'

docs = dict(
    data_frame=["A 'tidy' `pandas.DataFrame`"],
    x=[
        colref,
        "Values from this column are used to position marks along the x axis in cartesian coordinates.",
        "For horizontal `histogram`s, these values are used as inputs to `histfunc`.",
    ],
    y=[
        colref,
        "Values from this column are used to position marks along the y axis in cartesian coordinates.",
        "For vertical `histogram`s, these values are used as inputs to `histfunc`.",
    ],
    z=[
        colref,
        "Values from this column are used to position marks along the z axis in cartesian coordinates.",
        "For `density_heatmap` and `density_contour` these values are used as the inputs to `histfunc`.",
    ],
    a=[
        colref,
        "Values from this column are used to position marks along the a axis in ternary coordinates.",
    ],
    b=[
        colref,
        "Values from this column are used to position marks along the b axis in ternary coordinates.",
    ],
    c=[
        colref,
        "Values from this column are used to position marks along the c axis in ternary coordinates.",
    ],
    r=[
        colref,
        "Values from this column are used to position marks along the radial axis in polar coordinates.",
    ],
    theta=[
        colref,
        "Values from this column are used to position marks along the angular axis in polar coordinates.",
    ],
    lat=[
        colref,
        "Values from this column are used to position marks according to latitude on a map.",
    ],
    lon=[
        colref,
        "Values from this column are used to position marks according to longitude on a map.",
    ],
    locations=[
        colref,
        "Values from this column are be interpreted according to `locationmode` and mapped to longitude/latitude.",
    ],
    dimensions=[
        "(list of strings, names of columns in `data_frame`)",
        "Columns to be used in multidimensional visualization.",
    ],
    error_x=[
        colref,
        "Values from this column are used to size x-axis error bars.",
        "If `error_x_minus` is `None`, error bars will be symmetrical, otherwise `error_x` is used for the positive direction only.",
    ],
    error_x_minus=[
        colref,
        "Values from this column are used to size x-axis error bars in the negative direction.",
        "Ignored if `error_x` is `None`.",
    ],
    error_y=[
        colref,
        "Values from this column are used to size y-axis error bars.",
        "If `error_y_minus` is `None`, error bars will be symmetrical, otherwise `error_y` is used for the positive direction only.",
    ],
    error_y_minus=[
        colref,
        "Values from this column are used to size y-axis error bars in the negative direction.",
        "Ignored if `error_y` is `None`.",
    ],
    error_z=[
        colref,
        "Values from this column are used to size z-axis error bars.",
        "If `error_z_minus` is `None`, error bars will be symmetrical, otherwise `error_z` is used for the positive direction only.",
    ],
    error_z_minus=[
        colref,
        "Values from this column are used to size z-axis error bars in the negative direction.",
        "Ignored if `error_z` is `None`.",
    ],
    color=[colref, "Values from this column are used to assign color to marks."],
    opacity=["(number, between 0 and 1) Sets the opacity for markers."],
    line_dash=[
        colref,
        "Values from this column are used to assign dash-patterns to lines.",
    ],
    line_group=[
        colref,
        "Values from this column are used to group rows of `data_frame` into lines.",
    ],
    symbol=[colref, "Values from this column are used to assign symbols to marks."],
    size=[colref, "Values from this column are used to assign mark sizes."],
    hover_name=[colref, "Values from this column appear in bold in the hover tooltip."],
    hover_data=[
        colref_list,
        "Values from these columns appear as extra data in the hover tooltip.",
    ],
    text=[colref, "Values from this column appear in the figure as text labels."],
    locationmode=[
        "(string, one of 'ISO-3', 'USA-states', 'country names')",
        "Determines the set of locations used to match entries in `locations` to regions on the map.",
    ],
    facet_row=[
        colref,
        "Values from this column are used to assign marks to facetted subplots in the vertical direction.",
    ],
    facet_col=[
        colref,
        "Values from this column are used to assign marks to facetted subplots in the horizontal direction.",
    ],
    animation_frame=[
        colref,
        "Values from this column are used to assign marks to animation frames.",
    ],
    animation_group=[
        colref,
        "Values from this column are used to provide object-constancy across animation frames: rows with matching `animation_group`s will be treated as if they describe the same object in each frame.",
    ],
    symbol_sequence=[
        "(list of strings defining plotly.js symbols)",
        "When `symbol` is set, values in that column are assigned symbols by cycling through `symbol_sequence` in the order described in `category_orders`, unless the value of `symbol` is a key in `symbol_map`.",
    ],
    symbol_map=[
        "(dict with string keys and values that are strings defining plotly.js symbols, default `{}`)",
        "Used to override `symbol_sequence` to assign a specific symbols to marks corresponding with specific values.",
        "Keys in `symbol_map` should be values in the column denoted by `symbol`.",
    ],
    line_dash_map=[
        "(dict with string keys and values that are strings defining plotly.js dash-patterns, default `{}`)"
        "Used to override `line_dash_sequences` to assign a specific dash-patterns to lines corresponding with specific values.",
        "Keys in `line_dash_map` should be values in the column denoted by `line_dash`.",
    ],
    line_dash_sequence=[
        "(list of strings defining plotly.js dash-patterns)",
        "When `line_dash` is set, values in that column are assigned dash-patterns by cycling through `line_dash_sequence` in the order described in `category_orders`, unless the value of `line_dash` is a key in `line_dash_map`.",
    ],
    color_discrete_sequence=[
        "(list of valid CSS-color strings)",
        "When `color` is set and the values in the corresponding column are not numeric, values in that column are assigned colors by cycling through `color_discrete_sequence` in the order described in `category_orders`, unless the value of `color` is a key in `color_discrete_map`.",
        "Various useful color sequences are available in the `plotly_express.colors` submodules, specifically `plotly_express.colors.qualitative`.",
    ],
    color_discrete_map=[
        "(dict with string keys and values that are valid CSS-color strings, default `{}`)",
        "Used to override `color_discrete_sequence` to assign a specific colors to marks corresponding with specific values.",
        "Keys in `color_discrete_map` should be values in the column denoted by `color`.",
    ],
    color_continuous_scale=[
        "(list of valid CSS-color strings)",
        "This list is used to build a continuous color scale when the column denoted by `color` contains numeric data.",
        "Various useful color scales are available in the `plotly_express.colors` submodules, specifically `plotly_express.colors.sequential`, `plotly_express.colors.diverging` and `plotly_express.colors.cyclical`.",
    ],
    color_continuous_midpoint=[
        "(number, defaults to `None`)",
        "If set, computes the bounds of the continuous color scale to have the desired midpoint.",
        "Setting this value is recommended when using `plotly_express.colors.diverging` color scales as the inputs to `color_continuous_scale`.",
    ],
    size_max=["(integer, default 20)", "Set the maximum mark size when using `size`."],
    log_x=[
        "(boolean, default `False`)",
        "If `True`, the x-axis is log-scaled in cartesian coordinates.",
    ],
    log_y=[
        "(boolean, default `False`)",
        "If `True`, the y-axis is log-scaled in cartesian coordinates.",
    ],
    log_z=[
        "(boolean, default `False`)",
        "If `True`, the z-axis is log-scaled in cartesian coordinates.",
    ],
    log_r=[
        "(boolean, default `False`)",
        "If `True`, the radial axis is log-scaled in polar coordinates.",
    ],
    range_x=[
        "(2-element list of numbers)",
        "If provided, overrides auto-scaling on the x-axis in cartesian coordinates.",
    ],
    range_y=[
        "(2-element list of numbers)",
        "If provided, overrides auto-scaling on the y-axis in cartesian coordinates.",
    ],
    range_z=[
        "(2-element list of numbers)",
        "If provided, overrides auto-scaling on the z-axis in cartesian coordinates.",
    ],
    range_color=[
        "(2-element list of numbers)",
        "If provided, overrides auto-scaling on the continuous color scale.",
    ],
    range_r=[
        "(2-element list of numbers)",
        "If provided, overrides auto-scaling on the radial axis in polar coordinates.",
    ],
    title=["(string)", "The figure title."],
    template=[
        "(string or Plotly.py template object)",
        "The figure template name or definition.",
    ],
    width=["(integer, default `None`)", "The figure width in pixels."],
    height=["(integer, default `600`)", "The figure height in pixels."],
    labels=[
        "(dict with string keys and string values, default `{}`)",
        "By default, column names are used in the figure for axis titles, legend entries and hovers.",
        "This parameter allows this to be overridden.",
        "The keys of this dict should correspond to column names, and the values should correspond to the desired label to be displayed.",
    ],
    category_orders=[
        "(dict with string keys and list-of-string values, default `{}`)",
        "By default, in Python 3.6+, the order of categorical values in axes, legends and facets depends on the order in which these values are first encountered in `data_frame` (and no order is guaranteed by default in Python below 3.6).",
        "This parameter is used to force a specific ordering of values per column.",
        "The keys of this dict should correspond to column names, and the values should be lists of strings corresponding to the specific display order desired.",
    ],
    marginal=[
        "(string, one of `'rug'`, `'box'`, `'violin'`, `'histogram'`)",
        "If set, a subplot is drawn alongside the main plot, visulizing the distribution.",
    ],
    marginal_x=[
        "(string, one of `'rug'`, `'box'`, `'violin'`, `'histogram'`)",
        "If set, a horizontal subplot is drawn above the main plot, visulizing the x-distribution.",
    ],
    marginal_y=[
        "(string, one of `'rug'`, `'box'`, `'violin'`, `'histogram'`)",
        "If set, a vertical subplot is drawn to the right of the main plot, visulizing the y-distribution.",
    ],
    trendline=[
        "(string, one of `'ols'` or `'lowess'`, default `None`)",
        "If `'ols'`, an Ordinary Least Squares regression line will be drawn for each discrete-color/symbol group.",
        "If `'lowess`', a Locally Weighted Scatterplot Smoothing line will be drawn for each discrete-color/symbol group.",
    ],
    trendline_color_override=[
        "(string, valid CSS color)",
        "If provided, and if `trendline` is set, all trendlines will be drawn in this color.",
    ],
    render_mode=[
        "(string, one of `'auto'`, `'svg'` or `'webgl'`, default `'auto'`)",
        "Controls the browser API used to draw marks.",
        "`'svg`' is appropriate for figures of less than 1000 data points, and will allow for fully-vectorized output.",
        "`'webgl'` is likely necessary for acceptable performance above 1000 points but rasterizes part of the output. ",
        "`'auto'` uses heuristics to choose the mode.",
    ],
    direction=[
        "(string, one of '`counterclockwise'`, `'clockwise'`. Default is `'clockwise'`)",
        "Sets the direction in which increasing values of the angular axis are drawn.",
    ],
    start_angle=[
        "(integer, default is 90)",
        "Sets start angle for the angular axis, with 0 being due east and 90 being due north.",
    ],
    histfunc=[
        "(string, one of `'count'`, `'sum'`, `'avg'`, `'min'`, `'max'`. Default is `'count'`)"
        "Function used to aggregate values for summarization (note: can be normalized with `histnorm`).",
        "The arguments to this function for `histogram` are the values of `y` if `orientation` is `'v'`,",
        "otherwise the arguements are the values of `x`.",
        "The arguments to this function for `density_heatmap` and `density_contour` are the values of `z`.",
    ],
    histnorm=[
        "(string, one of `'percent'`, `'probability'`, `'density'`, `'probability density'`, default `None`)",
        "If `None`, the output of `histfunc` is used as is.",
        "If `'probability'`, the output of `histfunc` for a given bin is divided by the sum of the output of `histfunc` for all bins.",
        "If `'percent'`, the output of `histfunc` for a given bin is divided by the sum of the output of `histfunc` for all bins and multiplied by 100.",
        "If `'density'`, the output of `histfunc` for a given bin is divided by the size of the bin.",
        "If `'probability density'`, the output of `histfunc` for a given bin is normalized such that it corresponds to the probability that a random event whose distribution is described by the output of `histfunc` will fall into that bin.",
    ],
    barnorm=[
        "(string, one of `'fraction'` or `'percent'`, default is `None`)",
        "If set to `'fraction'`, the value of each bar is divided by the sum of all values at that location coordinate.",
        "`'percent'` is the same but multiplied by 100 to show percentages.",
    ],
    groupnorm=[
        "(string, one of `'fraction'` or `'percent'`, default is `None`)",
        "If set to `'fraction'`, the value of each point is divided by the sum of all values at that location coordinate.",
        "`'percent'` is the same but multiplied by 100 to show percentages.",
    ],
    barmode=[
        "(string, one of `'group'`, `'overlay'` or `'relative'`. Default is `'relative'`)",
        "In `'relative'` mode, bars are stacked above zero for positive values and below zero for negative values.",
        "In `'overlay'` mode, bars are on drawn top of one another.",
        "In `'group'` mode, bars are placed beside each other.",
    ],
    boxmode=[
        "(string, one of `'group'` or `'overlay'`. Default is `'group'`)",
        "In `'overlay'` mode, boxes are on drawn top of one another.",
        "In `'group'` mode, baxes are placed beside each other.",
    ],
    violinmode=[
        "(string, one of `'group'` or `'overlay'`. Default is `'group'`)",
        "In `'overlay'` mode, violins are on drawn top of one another.",
        "In `'group'` mode, violins are placed beside each other.",
    ],
    stripmode=[
        "(string, one of `'group'` or `'overlay'`. Default is `'group'`)",
        "In `'overlay'` mode, strips are on drawn top of one another.",
        "In `'group'` mode, strips are placed beside each other.",
    ],
    zoom=["(integer between 0 and 20, default is 8)", "Sets map zoom level."],
    orientation=[
        "(string, one of `'h'` for horizontal or `'v'` for vertical)",
        "Default is `'v'`.",
    ],
    line_close=[
        "(boolean, default `False`)",
        "If `True`, an extra line segment is drawn between the first and last point.",
    ],
    line_shape=["(string, one of `'linear'` or `'spline'`)", "Default is `'linear'`."],
    scope=[
        "(string, one of `'world'`, `'usa'`, `'europe'`, `'asia'`, `'africa'`, `'north america'`, `'south america'`)"
        "Default is `'world'` unless `projection` is set to `'albers usa'`, which forces `'usa'`."
    ],
    projection=[
        "(string, one of `'equirectangular'`, `'mercator'`, `'orthographic'`, `'natural earth'`, `'kavrayskiy7'`, `'miller'`, `'robinson'`, `'eckert4'`, `'azimuthal equal area'`, `'azimuthal equidistant'`, `'conic equal area'`, `'conic conformal'`, `'conic equidistant'`, `'gnomonic'`, `'stereographic'`, `'mollweide'`, `'hammer'`, `'transverse mercator'`, `'albers usa'`, `'winkel tripel'`, `'aitoff'`, `'sinusoidal'`)"
        "Default depends on `scope`."
    ],
    center=["(dict with `lat` and `lon` keys)", "Sets the center point of the map."],
    points=[
        "(string or boolean, one of `'all'`, `'outliers'`, or `False`. Default is `'outliers'`)",
        "If `'outliers'`, only the sample points lying outside the whiskers are shown.",
        "If `'all'`, all sample points are shown.",
        "If `False`, no sample points are shown",
    ],
    box=[
        "(boolean, default `False`)",
        "If `True`, boxes are drawn inside the violins.",
    ],
    notched=["(boolean, default `False`)", "If `True`, boxes are drawn with notches."],
    cumulative=[
        "(boolean, default `False`)",
        "If `True`, histogram values are cumulative.",
    ],
    nbins=["(positive integer)", "Sets the number of bins."],
    nbinsx=["(positive integer)", "Sets the number of bins along the x axis."],
    nbinsy=["(positive integer)", "Sets the number of bins along the y axis."],
)


def make_docstring(fn):
    result = (fn.__doc__ or "") + "\nArguments:\n"
    for arg in inspect.getargspec(fn)[0]:
        d = (
            " ".join(docs[arg] or "")
            if arg in docs
            else "(documentation missing from map)"
        )
        result += "    %s: %s\n" % (arg, d)
    result += "Returns:\n"
    result += "    A `Figure` object."
    return result



from ._orca import to_image, write_image
from . import orca

from ._json import to_json, from_json, read_json, write_json

from ._templates import templates, to_templated

from ._html import to_html, write_html

from ._renderers import renderers, show

from . import base_renderers



import _plotly_utils.basevalidators


class VisibleValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="visible", parent_name="pie", **kwargs):
        super(VisibleValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "info"),
            values=kwargs.pop("values", [True, False, "legendonly"]),
            **kwargs
        )


import _plotly_utils.basevalidators


class ValuessrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="valuessrc", parent_name="pie", **kwargs):
        super(ValuessrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ValuesValidator(_plotly_utils.basevalidators.DataArrayValidator):
    def __init__(self, plotly_name="values", parent_name="pie", **kwargs):
        super(ValuesValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "data"),
            **kwargs
        )


import _plotly_utils.basevalidators


class UirevisionValidator(_plotly_utils.basevalidators.AnyValidator):
    def __init__(self, plotly_name="uirevision", parent_name="pie", **kwargs):
        super(UirevisionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class UidValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="uid", parent_name="pie", **kwargs):
        super(UidValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            anim=kwargs.pop("anim", True),
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class TitleValidator(_plotly_utils.basevalidators.TitleValidator):
    def __init__(self, plotly_name="title", parent_name="pie", **kwargs):
        super(TitleValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Title"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            font
                Sets the font used for `title`. Note that the
                title's font used to be set by the now
                deprecated `titlefont` attribute.
            position
                Specifies the location of the `title`. Note
                that the title's position used to be set by the
                now deprecated `titleposition` attribute.
            text
                Sets the title of the chart. If it is empty, no
                title is displayed. Note that before the
                existence of `title.text`, the title's contents
                used to be defined as the `title` attribute
                itself. This behavior has been deprecated.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextsrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="textsrc", parent_name="pie", **kwargs):
        super(TextsrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextpositionsrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="textpositionsrc", parent_name="pie", **kwargs):
        super(TextpositionsrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextpositionValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="textposition", parent_name="pie", **kwargs):
        super(TextpositionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            values=kwargs.pop("values", ["inside", "outside", "auto", "none"]),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextinfoValidator(_plotly_utils.basevalidators.FlaglistValidator):
    def __init__(self, plotly_name="textinfo", parent_name="pie", **kwargs):
        super(TextinfoValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            extras=kwargs.pop("extras", ["none"]),
            flags=kwargs.pop("flags", ["label", "text", "value", "percent"]),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextfontValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="textfont", parent_name="pie", **kwargs):
        super(TextfontValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Textfont"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            color

            colorsrc
                Sets the source reference on plot.ly for  color
                .
            family
                HTML font family - the typeface that will be
                applied by the web browser. The web browser
                will only be able to apply a font if it is
                available on the system which it operates.
                Provide multiple font families, separated by
                commas, to indicate the preference in which to
                apply fonts if they aren't available on the
                system. The plotly service (at https://plot.ly
                or on-premise) generates images on a server,
                where only a select number of fonts are
                installed and supported. These include "Arial",
                "Balto", "Courier New", "Droid Sans",, "Droid
                Serif", "Droid Sans Mono", "Gravitas One", "Old
                Standard TT", "Open Sans", "Overpass", "PT Sans
                Narrow", "Raleway", "Times New Roman".
            familysrc
                Sets the source reference on plot.ly for
                family .
            size

            sizesrc
                Sets the source reference on plot.ly for  size
                .
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class TextValidator(_plotly_utils.basevalidators.DataArrayValidator):
    def __init__(self, plotly_name="text", parent_name="pie", **kwargs):
        super(TextValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "data"),
            **kwargs
        )


import _plotly_utils.basevalidators


class StreamValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="stream", parent_name="pie", **kwargs):
        super(StreamValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Stream"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            maxpoints
                Sets the maximum number of points to keep on
                the plots from an incoming stream. If
                `maxpoints` is set to 50, only the newest 50
                points will be displayed on the plot.
            token
                The stream id number links a data trace on a
                plot with a stream. See
                https://plot.ly/settings for more details.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class SortValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="sort", parent_name="pie", **kwargs):
        super(SortValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ShowlegendValidator(_plotly_utils.basevalidators.BooleanValidator):
    def __init__(self, plotly_name="showlegend", parent_name="pie", **kwargs):
        super(ShowlegendValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class ScalegroupValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="scalegroup", parent_name="pie", **kwargs):
        super(ScalegroupValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class RotationValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="rotation", parent_name="pie", **kwargs):
        super(RotationValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            max=kwargs.pop("max", 360),
            min=kwargs.pop("min", -360),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class PullsrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="pullsrc", parent_name="pie", **kwargs):
        super(PullsrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class PullValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="pull", parent_name="pie", **kwargs):
        super(PullValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "calc"),
            max=kwargs.pop("max", 1),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class OutsidetextfontValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="outsidetextfont", parent_name="pie", **kwargs):
        super(OutsidetextfontValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Outsidetextfont"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            color

            colorsrc
                Sets the source reference on plot.ly for  color
                .
            family
                HTML font family - the typeface that will be
                applied by the web browser. The web browser
                will only be able to apply a font if it is
                available on the system which it operates.
                Provide multiple font families, separated by
                commas, to indicate the preference in which to
                apply fonts if they aren't available on the
                system. The plotly service (at https://plot.ly
                or on-premise) generates images on a server,
                where only a select number of fonts are
                installed and supported. These include "Arial",
                "Balto", "Courier New", "Droid Sans",, "Droid
                Serif", "Droid Sans Mono", "Gravitas One", "Old
                Standard TT", "Open Sans", "Overpass", "PT Sans
                Narrow", "Raleway", "Times New Roman".
            familysrc
                Sets the source reference on plot.ly for
                family .
            size

            sizesrc
                Sets the source reference on plot.ly for  size
                .
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class OpacityValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="opacity", parent_name="pie", **kwargs):
        super(OpacityValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "style"),
            max=kwargs.pop("max", 1),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class NameValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="name", parent_name="pie", **kwargs):
        super(NameValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class MetasrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="metasrc", parent_name="pie", **kwargs):
        super(MetasrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class MetaValidator(_plotly_utils.basevalidators.AnyValidator):
    def __init__(self, plotly_name="meta", parent_name="pie", **kwargs):
        super(MetaValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class MarkerValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="marker", parent_name="pie", **kwargs):
        super(MarkerValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Marker"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            colors
                Sets the color of each sector. If not
                specified, the default trace color set is used
                to pick the sector colors.
            colorssrc
                Sets the source reference on plot.ly for
                colors .
            line
                plotly.graph_objects.pie.marker.Line instance
                or dict with compatible properties
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class LegendgroupValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="legendgroup", parent_name="pie", **kwargs):
        super(LegendgroupValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class LabelssrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="labelssrc", parent_name="pie", **kwargs):
        super(LabelssrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class LabelsValidator(_plotly_utils.basevalidators.DataArrayValidator):
    def __init__(self, plotly_name="labels", parent_name="pie", **kwargs):
        super(LabelsValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "data"),
            **kwargs
        )


import _plotly_utils.basevalidators


class Label0Validator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="label0", parent_name="pie", **kwargs):
        super(Label0Validator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class InsidetextfontValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="insidetextfont", parent_name="pie", **kwargs):
        super(InsidetextfontValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Insidetextfont"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            color

            colorsrc
                Sets the source reference on plot.ly for  color
                .
            family
                HTML font family - the typeface that will be
                applied by the web browser. The web browser
                will only be able to apply a font if it is
                available on the system which it operates.
                Provide multiple font families, separated by
                commas, to indicate the preference in which to
                apply fonts if they aren't available on the
                system. The plotly service (at https://plot.ly
                or on-premise) generates images on a server,
                where only a select number of fonts are
                installed and supported. These include "Arial",
                "Balto", "Courier New", "Droid Sans",, "Droid
                Serif", "Droid Sans Mono", "Gravitas One", "Old
                Standard TT", "Open Sans", "Overpass", "PT Sans
                Narrow", "Raleway", "Times New Roman".
            familysrc
                Sets the source reference on plot.ly for
                family .
            size

            sizesrc
                Sets the source reference on plot.ly for  size
                .
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class IdssrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="idssrc", parent_name="pie", **kwargs):
        super(IdssrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class IdsValidator(_plotly_utils.basevalidators.DataArrayValidator):
    def __init__(self, plotly_name="ids", parent_name="pie", **kwargs):
        super(IdsValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            anim=kwargs.pop("anim", True),
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "data"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HovertextsrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="hovertextsrc", parent_name="pie", **kwargs):
        super(HovertextsrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HovertextValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="hovertext", parent_name="pie", **kwargs):
        super(HovertextValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HovertemplatesrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="hovertemplatesrc", parent_name="pie", **kwargs):
        super(HovertemplatesrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HovertemplateValidator(_plotly_utils.basevalidators.StringValidator):
    def __init__(self, plotly_name="hovertemplate", parent_name="pie", **kwargs):
        super(HovertemplateValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HoverlabelValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="hoverlabel", parent_name="pie", **kwargs):
        super(HoverlabelValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Hoverlabel"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            align
                Sets the horizontal alignment of the text
                content within hover label box. Has an effect
                only if the hover label text spans more two or
                more lines
            alignsrc
                Sets the source reference on plot.ly for  align
                .
            bgcolor
                Sets the background color of the hover labels
                for this trace
            bgcolorsrc
                Sets the source reference on plot.ly for
                bgcolor .
            bordercolor
                Sets the border color of the hover labels for
                this trace.
            bordercolorsrc
                Sets the source reference on plot.ly for
                bordercolor .
            font
                Sets the font used in hover labels.
            namelength
                Sets the default length (in number of
                characters) of the trace name in the hover
                labels for all traces. -1 shows the whole name
                regardless of length. 0-3 shows the first 0-3
                characters, and an integer >3 will show the
                whole name if it is less than that many
                characters, but if it is longer, will truncate
                to `namelength - 3` characters and add an
                ellipsis.
            namelengthsrc
                Sets the source reference on plot.ly for
                namelength .
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class HoverinfosrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="hoverinfosrc", parent_name="pie", **kwargs):
        super(HoverinfosrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HoverinfoValidator(_plotly_utils.basevalidators.FlaglistValidator):
    def __init__(self, plotly_name="hoverinfo", parent_name="pie", **kwargs):
        super(HoverinfoValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", True),
            edit_type=kwargs.pop("edit_type", "none"),
            extras=kwargs.pop("extras", ["all", "none", "skip"]),
            flags=kwargs.pop("flags", ["label", "text", "value", "percent", "name"]),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class HoleValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="hole", parent_name="pie", **kwargs):
        super(HoleValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            max=kwargs.pop("max", 1),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class DomainValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="domain", parent_name="pie", **kwargs):
        super(DomainValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Domain"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            column
                If there is a layout grid, use the domain for
                this column in the grid for this pie trace .
            row
                If there is a layout grid, use the domain for
                this row in the grid for this pie trace .
            x
                Sets the horizontal domain of this pie trace
                (in plot fraction).
            y
                Sets the vertical domain of this pie trace (in
                plot fraction).
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class DlabelValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="dlabel", parent_name="pie", **kwargs):
        super(DlabelValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class DirectionValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="direction", parent_name="pie", **kwargs):
        super(DirectionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "style"),
            values=kwargs.pop("values", ["clockwise", "counterclockwise"]),
            **kwargs
        )


import _plotly_utils.basevalidators


class CustomdatasrcValidator(_plotly_utils.basevalidators.SrcValidator):
    def __init__(self, plotly_name="customdatasrc", parent_name="pie", **kwargs):
        super(CustomdatasrcValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "none"),
            role=kwargs.pop("role", "info"),
            **kwargs
        )


import _plotly_utils.basevalidators


class CustomdataValidator(_plotly_utils.basevalidators.DataArrayValidator):
    def __init__(self, plotly_name="customdata", parent_name="pie", **kwargs):
        super(CustomdataValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "calc"),
            role=kwargs.pop("role", "data"),
            **kwargs
        )



import _plotly_utils.basevalidators


class SymbolValidator(_plotly_utils.basevalidators.EnumeratedValidator):
    def __init__(self, plotly_name="symbol", parent_name="violin.marker", **kwargs):
        super(SymbolValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=kwargs.pop("array_ok", False),
            edit_type=kwargs.pop("edit_type", "plot"),
            role=kwargs.pop("role", "style"),
            values=kwargs.pop(
                "values",
                [
                    0,
                    "circle",
                    100,
                    "circle-open",
                    200,
                    "circle-dot",
                    300,
                    "circle-open-dot",
                    1,
                    "square",
                    101,
                    "square-open",
                    201,
                    "square-dot",
                    301,
                    "square-open-dot",
                    2,
                    "diamond",
                    102,
                    "diamond-open",
                    202,
                    "diamond-dot",
                    302,
                    "diamond-open-dot",
                    3,
                    "cross",
                    103,
                    "cross-open",
                    203,
                    "cross-dot",
                    303,
                    "cross-open-dot",
                    4,
                    "x",
                    104,
                    "x-open",
                    204,
                    "x-dot",
                    304,
                    "x-open-dot",
                    5,
                    "triangle-up",
                    105,
                    "triangle-up-open",
                    205,
                    "triangle-up-dot",
                    305,
                    "triangle-up-open-dot",
                    6,
                    "triangle-down",
                    106,
                    "triangle-down-open",
                    206,
                    "triangle-down-dot",
                    306,
                    "triangle-down-open-dot",
                    7,
                    "triangle-left",
                    107,
                    "triangle-left-open",
                    207,
                    "triangle-left-dot",
                    307,
                    "triangle-left-open-dot",
                    8,
                    "triangle-right",
                    108,
                    "triangle-right-open",
                    208,
                    "triangle-right-dot",
                    308,
                    "triangle-right-open-dot",
                    9,
                    "triangle-ne",
                    109,
                    "triangle-ne-open",
                    209,
                    "triangle-ne-dot",
                    309,
                    "triangle-ne-open-dot",
                    10,
                    "triangle-se",
                    110,
                    "triangle-se-open",
                    210,
                    "triangle-se-dot",
                    310,
                    "triangle-se-open-dot",
                    11,
                    "triangle-sw",
                    111,
                    "triangle-sw-open",
                    211,
                    "triangle-sw-dot",
                    311,
                    "triangle-sw-open-dot",
                    12,
                    "triangle-nw",
                    112,
                    "triangle-nw-open",
                    212,
                    "triangle-nw-dot",
                    312,
                    "triangle-nw-open-dot",
                    13,
                    "pentagon",
                    113,
                    "pentagon-open",
                    213,
                    "pentagon-dot",
                    313,
                    "pentagon-open-dot",
                    14,
                    "hexagon",
                    114,
                    "hexagon-open",
                    214,
                    "hexagon-dot",
                    314,
                    "hexagon-open-dot",
                    15,
                    "hexagon2",
                    115,
                    "hexagon2-open",
                    215,
                    "hexagon2-dot",
                    315,
                    "hexagon2-open-dot",
                    16,
                    "octagon",
                    116,
                    "octagon-open",
                    216,
                    "octagon-dot",
                    316,
                    "octagon-open-dot",
                    17,
                    "star",
                    117,
                    "star-open",
                    217,
                    "star-dot",
                    317,
                    "star-open-dot",
                    18,
                    "hexagram",
                    118,
                    "hexagram-open",
                    218,
                    "hexagram-dot",
                    318,
                    "hexagram-open-dot",
                    19,
                    "star-triangle-up",
                    119,
                    "star-triangle-up-open",
                    219,
                    "star-triangle-up-dot",
                    319,
                    "star-triangle-up-open-dot",
                    20,
                    "star-triangle-down",
                    120,
                    "star-triangle-down-open",
                    220,
                    "star-triangle-down-dot",
                    320,
                    "star-triangle-down-open-dot",
                    21,
                    "star-square",
                    121,
                    "star-square-open",
                    221,
                    "star-square-dot",
                    321,
                    "star-square-open-dot",
                    22,
                    "star-diamond",
                    122,
                    "star-diamond-open",
                    222,
                    "star-diamond-dot",
                    322,
                    "star-diamond-open-dot",
                    23,
                    "diamond-tall",
                    123,
                    "diamond-tall-open",
                    223,
                    "diamond-tall-dot",
                    323,
                    "diamond-tall-open-dot",
                    24,
                    "diamond-wide",
                    124,
                    "diamond-wide-open",
                    224,
                    "diamond-wide-dot",
                    324,
                    "diamond-wide-open-dot",
                    25,
                    "hourglass",
                    125,
                    "hourglass-open",
                    26,
                    "bowtie",
                    126,
                    "bowtie-open",
                    27,
                    "circle-cross",
                    127,
                    "circle-cross-open",
                    28,
                    "circle-x",
                    128,
                    "circle-x-open",
                    29,
                    "square-cross",
                    129,
                    "square-cross-open",
                    30,
                    "square-x",
                    130,
                    "square-x-open",
                    31,
                    "diamond-cross",
                    131,
                    "diamond-cross-open",
                    32,
                    "diamond-x",
                    132,
                    "diamond-x-open",
                    33,
                    "cross-thin",
                    133,
                    "cross-thin-open",
                    34,
                    "x-thin",
                    134,
                    "x-thin-open",
                    35,
                    "asterisk",
                    135,
                    "asterisk-open",
                    36,
                    "hash",
                    136,
                    "hash-open",
                    236,
                    "hash-dot",
                    336,
                    "hash-open-dot",
                    37,
                    "y-up",
                    137,
                    "y-up-open",
                    38,
                    "y-down",
                    138,
                    "y-down-open",
                    39,
                    "y-left",
                    139,
                    "y-left-open",
                    40,
                    "y-right",
                    140,
                    "y-right-open",
                    41,
                    "line-ew",
                    141,
                    "line-ew-open",
                    42,
                    "line-ns",
                    142,
                    "line-ns-open",
                    43,
                    "line-ne",
                    143,
                    "line-ne-open",
                    44,
                    "line-nw",
                    144,
                    "line-nw-open",
                ],
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class SizeValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="size", parent_name="violin.marker", **kwargs):
        super(SizeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            anim=kwargs.pop("anim", True),
            array_ok=kwargs.pop("array_ok", False),
            edit_type=kwargs.pop("edit_type", "calc"),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class OutliercolorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(
        self, plotly_name="outliercolor", parent_name="violin.marker", **kwargs
    ):
        super(OutliercolorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class OpacityValidator(_plotly_utils.basevalidators.NumberValidator):
    def __init__(self, plotly_name="opacity", parent_name="violin.marker", **kwargs):
        super(OpacityValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            anim=kwargs.pop("anim", True),
            array_ok=kwargs.pop("array_ok", False),
            edit_type=kwargs.pop("edit_type", "style"),
            max=kwargs.pop("max", 1),
            min=kwargs.pop("min", 0),
            role=kwargs.pop("role", "style"),
            **kwargs
        )


import _plotly_utils.basevalidators


class LineValidator(_plotly_utils.basevalidators.CompoundValidator):
    def __init__(self, plotly_name="line", parent_name="violin.marker", **kwargs):
        super(LineValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop("data_class_str", "Line"),
            data_docs=kwargs.pop(
                "data_docs",
                """
            color
                Sets themarker.linecolor. It accepts either a
                specific color or an array of numbers that are
                mapped to the colorscale relative to the max
                and min values of the array or relative to
                `marker.line.cmin` and `marker.line.cmax` if
                set.
            outliercolor
                Sets the border line color of the outlier
                sample points. Defaults to marker.color
            outlierwidth
                Sets the border line width (in px) of the
                outlier sample points.
            width
                Sets the width (in px) of the lines bounding
                the marker points.
""",
            ),
            **kwargs
        )


import _plotly_utils.basevalidators


class ColorValidator(_plotly_utils.basevalidators.ColorValidator):
    def __init__(self, plotly_name="color", parent_name="violin.marker", **kwargs):
        super(ColorValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            anim=kwargs.pop("anim", True),
            array_ok=kwargs.pop("array_ok", False),
            edit_type=kwargs.pop("edit_type", "style"),
            role=kwargs.pop("role", "style"),
            **kwargs
        )
